#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <math.h>
#include <sndfile.h>

static int guess_direction (const char * filename1, const char * filename2) ;
static int guess_major_format (const char * filename) ;
static void generate_pcm(const char * wavfilename, const char * pcmfilename);
static void generate_wav(const char * pcmfilename, const char * wavfilename);
static void print_usage_on_error(void);

int
main (int argc, char * argv [])
{
	const char * filename1 ;
	const char * filename2 ;
	int wav2pcm = 0;

	if(argc != 3){
	  print_usage_on_error();
	}

	filename1 = argv [1] ;
	filename2 = argv [2] ;
	
	wav2pcm = guess_direction(filename1, filename2);

	if(wav2pcm)
	  generate_pcm(filename1, filename2);
	else
	  generate_wav(filename1, filename2);

	return 0 ;
}

static void
generate_wav(const char * pcmfilename, const char * wavfilename)
{
  char outfilename[2048];
  SNDFILE * outfile ;
  SNDFILE * wavfile ;
  SNDFILE * pcmfile ;
  SF_INFO wavinfo ;
  SF_INFO pcminfo ;
  SF_INFO outinfo ;
  int buff[2];

  memset(outfilename, 0, sizeof(outfilename));
  sprintf(outfilename, "%s.wav", pcmfilename);
  
  memset (&wavinfo, 0, sizeof (wavinfo)) ;
  memset (&pcminfo, 0, sizeof (pcminfo)) ;
  memset (&outinfo, 0, sizeof (outinfo)) ;
  
  wavfile = sf_open(wavfilename, SFM_READ, &wavinfo);

  if (wavfile == NULL){      
    printf ("\nERROR : Not able to open wav file named '%s' : %s/\n", wavfilename, sf_strerror (NULL)) ;
    exit (1) ;
  } ;


  if ((!wavinfo.format & SF_FORMAT_PCM_16) || (!wavinfo.channels == 2)){
    printf("\nERROR : .wav file must be SF_FORMAT_PCM_16 in stereo\n");
  }

  pcminfo.format = SF_FORMAT_RAW | SF_FORMAT_PCM_16;
  pcminfo.samplerate = wavinfo.samplerate;
  pcminfo.channels = wavinfo.channels;

  pcmfile = sf_open(pcmfilename, SFM_READ, &pcminfo);

  if (pcmfile == NULL){      
    printf ("\nERROR : Not able to open pcm file named '%s' : %s/\n", pcmfilename, sf_strerror (NULL)) ;
    exit (1) ;
  } ;

  outinfo.format = SF_FORMAT_WAV | SF_FORMAT_PCM_16;
  outinfo.samplerate = wavinfo.samplerate;
  outinfo.channels = wavinfo.channels;

  outfile = sf_open(outfilename, SFM_WRITE, &outinfo);
  
  if (outfile == NULL){      
    printf ("\nERROR : Not able to create wav file named '%s' : %s/\n", outfilename, sf_strerror (NULL)) ;
    exit (1) ;
  } ;

  while(sf_read_int(pcmfile, buff, 2)==2){
    if(sf_write_int(outfile, buff, 2) != 2){
      printf("\nERROR : unable to write to '%s' : %s/\n", outfilename, sf_strerror(NULL));
    }
  }
  
  sf_close (wavfile) ;
  sf_close (outfile) ;
  sf_close (pcmfile) ;
  
}

static void
generate_pcm (const char * wavfilename, const char * pcmfilename)
{
	SNDFILE * wavfile ;
	SNDFILE * pcmfile ;
	SF_INFO wavinfo ;
	SF_INFO pcminfo ;
	int buff[2];

	memset (&wavinfo, 0, sizeof (wavinfo)) ;
	memset (&pcminfo, 0, sizeof (pcminfo)) ;

	wavfile = sf_open (wavfilename, SFM_READ, &wavinfo) ;

	if (wavfile == NULL){      
	  printf ("\nERROR : Not able to open wav file named '%s' : %s/\n", wavfilename, sf_strerror (NULL)) ;
	  exit (1) ;
	} ;

	pcminfo.format = SF_FORMAT_RAW | SF_FORMAT_PCM_16;
	pcminfo.samplerate = wavinfo.samplerate;
	pcminfo.channels = wavinfo.channels;

	if ((!wavinfo.format & SF_FORMAT_PCM_16) || (!wavinfo.channels == 2)){
	  printf("\nERROR : .wav file must be SF_FORMAT_PCM_16 in stereo\n");
	}

	pcmfile = sf_open (pcmfilename, SFM_WRITE, &pcminfo) ;

	if (pcmfile == NULL){      
	  printf ("\nERROR : Not able to create pcm file named '%s' : %s/\n", pcmfilename, sf_strerror (NULL)) ;
	  exit (1) ;
	} ;
	
	while(sf_read_int(wavfile, buff, 2)==2){
	  if(sf_write_int(pcmfile, buff, 2) != 2){
	    printf("\nERROR : unable to write to '%s' : %s/\n", pcmfilename, sf_strerror(NULL));
	  }
	}

	sf_close (wavfile) ;
	sf_close (pcmfile) ;
} 


static int 
guess_direction(const char * filename1, const char* filename2)
{
  int format1;
  int format2;

  format1 = guess_major_format(filename1);
  format2 = guess_major_format(filename2);
  
  if (format1 == SF_FORMAT_WAV && format2 == SF_FORMAT_RAW)
    return 1;
  if (format2 == SF_FORMAT_WAV && format1 == SF_FORMAT_RAW)
    return 0;

  print_usage_on_error();
  return 0;
}

static int
guess_major_format (const char * filename)
{
	const char * ext ;

	ext = strrchr (filename, '.') ;
	
	if (ext == NULL){	
	  printf ("\nERROR : cannot figure out file type from file name '%s'.\n\n", filename) ;
	  exit (1) ;
	} ;

	if (strcasecmp (ext, ".wav") == 0)
		return SF_FORMAT_WAV ;
	if (strcasecmp (ext, ".pcm") == 0)
		return SF_FORMAT_RAW ;

	printf ("\nERROR : Can only convert files with extentions 'wav', 'pcm'.\n\n") ;
	exit (1) ;
	return 0 ;
} 

static void
print_usage_on_error()
{
  printf("\n");
  printf("usage: \n");
  printf("\n");
  printf("sndfile-wav-util foo.wav bar.pcm  will generate a raw pcm file from foo.wav and write it to bar.pcm\n");
  printf("\n");
  printf("sndfile-wav-util bar.pcm foo.wav  will generate a .wav file 'bar.pcm.wav' from bar.pcm, at the sampling rate used in foo.wav\n");
  printf("\n");
  exit(1);
}
